#include "sp_mathlin.h"
#include "sp_math.h"



int IsEqual(float a,float b)
{
  if(a!=0)
    {
      if( fabs((a-b)/a) < 00.1 )
        {return 1; }
      else
        {return 0; }
    }
  else if(b!=0)
        {
      if( fabs((a-b)/b) < 00.1 )
        return 1;
      else
        return 0;
    }
  else
    {
      return 1;
    }
}


/* ************************ Sp_Vecteur *************************** */



Sp_Vecteur::Sp_Vecteur()
{
  // par securite
  x=y=z=w=0;
}

Sp_Vecteur::Sp_Vecteur(float x1,float y1,float z1)
{
  x=x1; y=y1; z=z1; w=0;
}

Sp_Vecteur::Sp_Vecteur(float x1,float y1,float z1,float w1)
{
  x=x1; y=y1; z=z1; w=w1;
}

Sp_Vecteur::Sp_Vecteur(const Sp_Vecteur &v)
{
  x=v.x; y=v.y; z=v.z; w=v.w;
}

Sp_Vecteur &Sp_Vecteur::operator=(const Sp_Vecteur &v)
{
  x=v.x; y=v.y; z=v.z; w=v.w;
  return *this;
}

Sp_Vecteur Sp_Vecteur::operator+(const Sp_Vecteur &a) const
{
  Sp_Vecteur r;
  r.x=x+a.x;
  r.y=y+a.y;
  r.z=z+a.z;
  r.w=w+a.w;
  return r;
}

Sp_Vecteur Sp_Vecteur::operator-(const Sp_Vecteur &a) const
{
  Sp_Vecteur r;
  r.x=x-a.x;
  r.y=y-a.y;
  r.z=z-a.z;
  r.w=w-a.w;
  return r;
}

float      Sp_Vecteur::operator*(const Sp_Vecteur &a) const
{
  return x*a.x+y*a.y+z*a.z+w*a.w;
}

Sp_Vecteur Sp_Vecteur::operator*(const float &c) const
{
  Sp_Vecteur r;
  r.x=x*c; r.y=y*c; r.z=z*c; r.w=w*c;
  return r;
}


Sp_Vecteur Sp_Vecteur::operator^(const Sp_Vecteur &v) const
{
  Sp_Vecteur r;
  r.x=y*v.z-z*v.y; 
  r.y=z*v.x-x*v.z; 
  r.z=x*v.y-y*v.x; 
  return r;
}

void Sp_Vecteur::Norme()
{
  float norme;
  norme=sqrt(x*x+y*y+z*z);
  if(norme!=0)
    {
      x/=norme; y/=norme; z/=norme;
    }
  else
    {
      printf("it is not a good idea to norme a null vector\n");
    }
}

float Sp_Vecteur::Compute_norme() const
{
  return sqrt(x*x+y*y+z*z);
}


int Sp_Vecteur::IsNull() const
{
  if((x==0)&&(y==0)&&(z==0))
    return 1;
  return 0;
}


void Sp_Vecteur::Print() const
{
  //printf("x:%f y:%f z:%f w:%f\n",x,y,z,w);
}

/* ************************ Sp_Point ***************************** */

Sp_Point::Sp_Point()
{}

Sp_Point::Sp_Point(float x1,float y1,float z1)
{
  x=x1; y=y1; z=z1; w=0;
}

Sp_Point::Sp_Point(float x1,float y1,float z1,float w1)
{
  x=x1; y=y1; z=z1; w=w1;
}


Sp_Point::Sp_Point(const Sp_Point &p)
{
 x=p.x; y=p.y; z=p.z; w=p.w;
}


Sp_Point &Sp_Point::operator=(const Sp_Point &v)
{
  this->x=v.x; this->y=v.y; this->z=v.z; this->w=v.w;
  return *this;
}


Sp_Point Sp_Point::operator+(const Sp_Vecteur &v) const
{
  Sp_Point r;
  r.x=x+v.x; r.y=y+v.y;
  r.z=z+v.z; r.w=w+v.w;
  return r;
}

Sp_Point Sp_Point::operator-(const Sp_Vecteur &v) const
{
  Sp_Point r;
  r.x=x-v.x; r.y=y-v.y;
  r.z=z-v.z; r.w=w-v.w;
  return r;
}

Sp_Vecteur Sp_Point::operator-(const Sp_Point &p) const 
{
  Sp_Vecteur v;
  v.x=x-p.x; v.y=y-p.y;
  v.z=z-p.z; v.w=w-p.w;
  return v;
}

//int Sp_Point::operator==(const Sp_Point &p) const
//{
//    if((x==p.x)&&(y==p.y)&&(z==p.z))
//    return 1;
//    else
//    return 0;
//}


float Sp_Point::Distance(const Sp_Point &point) const
{
    Sp_Point tmp;
    Sp_Vecteur tmp2;
    tmp.x=this->x;tmp.y=this->y;tmp.z=this->z;
    tmp2=tmp-point;
    return tmp2.Compute_norme();
}


void Sp_Point::Print() const
{
  printf("x:%f y:%f z:%f w:%f\n",x,y,z,w);
}


Sp_Droite::Sp_Droite(Sp_Point &p0,Sp_Vecteur &v0)
{
  p.x=p0.x;p.y=p0.y;p.z=p0.z;p.w=p0.w; 
  v.x=v0.x;v.y=v0.y;v.z=v0.z;v.w=v0.w;
}

Sp_Droite::Sp_Droite(Sp_Point &p0,Sp_Point &p1)
{
  p.x=p0.x;p.y=p0.y;p.z=p0.z;p.w=p0.w;
  Sp_Vecteur v0=p0-p1;
  v0.Norme();
  v.x=v0.x;v.y=v0.y;v.z=v0.z;v.w=v0.w;
}

void Sp_Droite::Print() const 
{
  printf("une droite: un point et un vecteur\n");
  p.Print();
  v.Print();
}

int Sp_Droite::IsIn(const Sp_Point &P1) const
{
  // lambda 
  float lx,ly,lz;
  // Is lambda existing ?
  int   fx,fy,fz;
  // coord X
  if(v.x!=0)
    {
      fx=1;
      lx=(P1.x-p.x)/v.x;
    }
  else
    {
      fx=0;
    }
  // coord Y
  if(v.y!=0)
    {
      fy=1;
      ly=(P1.y-p.y)/v.y;
    }
  else
    {
      fy=0;
    }
  // coord Z
  if(v.z!=0)
    {
      fz=1;
      lz=(P1.z-p.z)/v.z;
    }
  else
    {
      fz=0;
    }
  // we are testing if l? are equal
  if((fx!=0)&&(fy!=0))
    {
      if(IsEqual(lx,ly)==0)
        return 0;
    }
  if((fy!=0)&&(fz!=0))
    {
      if(IsEqual(ly,lz)==0)
        return 0;
    }
  if((fx!=0)&&(fz!=0))
    {
      if(IsEqual(lx,lz)==0)
        return 0;
    }
  if((fx==0)&&(fy==0)&&(fz==0))
    {
      printf("pb le vecteur directeur d une droite est nul\n");
      return 0;
    }
  return 1;
}


Sp_Segment::Sp_Segment(Sp_Point *pp0,Sp_Point *pp1)
{
  if((pp0->x==pp1->x) && (pp0->y==pp1->y) && (pp0->z==pp1->z))
    printf("ceci n est pas un segment: on a deux points identiques\n");
  p1=pp0;
  p2=pp1;
}
Sp_Segment::Sp_Segment(Sp_Point &pp0,Sp_Point &pp1)
{
  if((pp0.x==pp1.x) && (pp0.y==pp1.y) && (pp0.z==pp1.z))
    printf("ceci n est pas un segment: on a deux points identiques\n");
  p1=&pp0;
  p2=&pp1;
}


int Sp_Segment::IsBetween(const Sp_Point &P1) const
{
    Sp_Point &A=*p1;
    Sp_Point &B=*p2;
    Sp_Vecteur AB=B-A;
    Sp_Vecteur AP1=P1-A;
    Sp_Vecteur P1B=B-P1;
    if(AP1.IsNull()!=0)
        return 1;
    if(P1B.IsNull()!=0)
        return 1;
    AB.Norme(); AP1.Norme(); P1B.Norme();
    float c1,c2;
    c1=AB*AP1;
    c2=AB*P1B;
    if((c1>=0)&&(c2>=0))
        return 1;
    else
        return 0;
}



int Sp_Segment::IsIn(const Sp_Point &P1) const
{
    Sp_Point &A=*p1;
    Sp_Point &B=*p2;
    
    Sp_Vecteur AB=B-A;
    Sp_Vecteur AP1=P1-A;
    Sp_Vecteur P1B=B-P1;
    if(AP1.IsNull()!=0)
        return 1;
    if(P1B.IsNull()!=0)
        return 1;
    AB.Norme(); AP1.Norme(); P1B.Norme();
    float c1,c2;
    c1=AB*AP1;
    c2=AB*P1B;
    printf("c1:%f c2:%f\n",c1,c2);
    if((c1>=0.99)&&(c1<1.001))
    {
        if((c2>=0.99)&&(c2<1.001))
            return 1;
    }
    return 0;
}

int Sp_Segment::intersection(const Sp_Segment &seg,const Sp_Vecteur &normale,Sp_Point &resultat) const
{
    
        // we are computing the intersection of two segments
        // we are creating a plane
    Sp_Point PP1(*p1);
    Sp_Point PP2(*p2);
    Sp_Plan plan(PP1,PP2,normale);
    Sp_Droite droite(*seg.p1,*seg.p2);
    Intersection inter(droite,plan);
        // we are testing if we have an intersection
    if(inter.liste_points.size()==0)
        return 0;
        // we are testing if the result is belonging to the segment
        //printf("alors la il doit avoir une intersection\n");
    typedef list<Sp_Point>::iterator iter;
    iter i=inter.liste_points.begin();
    Sp_Point result=(*i);
    if((IsBetween(result)!=0)&&(seg.IsBetween(result)!=0)){
            //printf("alors la il doit avoir une intersection sur!!!\n");
        resultat=result;
        return 1;
    }
    printf(" il y a un putain de pb %d %d\n",IsIn(result),seg.IsIn(result));    
    printf("intersection de deux seg\n");
    printf("seg1\n");
    Print();
    printf("seg2\n");
    seg.Print();
    printf("result");
    result.Print();
    return 0;
}


void Sp_Segment::Print() const
{
    printf("un segement\n");
    p1->Print();
    p2->Print();
}



Sp_Plan::Sp_Plan(float a,float b,float c,float d)
{
  A=a; B=b;
  C=c; D=d;
}

Sp_Plan::Sp_Plan(const Sp_Plan &p)
{
  A=p.A; B=p.B;
  C=p.C; D=p.D;
}

Sp_Plan::Sp_Plan(Sp_Point &pt,Sp_Vecteur &normale)
{
A=normale.x;
B=normale.y;
C=normale.z;
D=-(normale.x*pt.x)-(normale.y*pt.y)-(normale.z*pt.z);
    if((A==0)&&(B==0)&&(C==0))
        exit(2);

}

Sp_Plan::Sp_Plan(const Sp_Point &pt,const Sp_Point &p2,const Sp_Vecteur &v0)
{

    Sp_Vecteur v1=p2-pt;
    Sp_Vecteur normale=v0^v1;
    A=normale.x;
    B=normale.y;
    C=normale.z;
    if((A==0)&&(B==0)&&(C==0)){
            // printf("lolo\n");
        
    }
    
    D=-(normale.x*pt.x)-(normale.y*pt.y)-(normale.z*pt.z);
  
}

Sp_Vecteur Sp_Plan::Normale() const
{
//    if((A==0)&&(B==0)&&(C==0))
//        exit(4);
    
    Sp_Vecteur v;
    v.x=A; v.y=B; v.z=C; v.w=0;
    return v;
}

float Sp_Plan::Sign(const Sp_Point &p) const
{
  return A*p.x+B*p.y+C*p.z+D;
}

// Project a point on the plan
Sp_Point Sp_Plan::Project(const Sp_Point &p) const
{
 float lambda;
 float K=Sign(p);
 lambda=K/(A*A+B*B+C*C);
 Sp_Point P=p-(Normale()*lambda);
 return P;
}

Sp_Vecteur Sp_Plan::Project(const Sp_Vecteur &i1) const
{
    printf("****************\n");
    Sp_Vecteur normale=Normale();
    i1.Print();
    normale.x=A;normale.y=B;normale.z=C;
    normale.Print();
    Sp_Vecteur k1=i1^Normale();
    k1.Print();
    
    return Normale()^k1;
    
}

float Sp_Plan::Distance(const Sp_Point &p) const
{
  float lambda;
  float K=Sign(p);
  lambda=fabs(K/(A*A+B*B+C*C));
  return lambda*Normale().Compute_norme();
}
                        
Sp_Sphere::Sp_Sphere(float x,float y,float z,float r0)
{
  xo=x; yo=y; zo=z; r=r0;
}

Sp_Sphere::Sp_Sphere(Sp_Point &p,float r0)
{
  xo=p.x; yo=p.y; zo=p.z; r=r0;
}

