#ifndef SP_MATH_H
#define SP_MATH_H

#include <stdio.h>
#include <stdlib.h>
#include <list>
#include <math.h>
using namespace std;


class Sp_Vecteur;
class Sp_Point;
class Sp_Droite;
class Sp_Plan;
class Sp_Sphere;


class Sp_Vecteur
{
 public:
  // les constructeurs
  Sp_Vecteur();
  Sp_Vecteur(float x1,float y1,float z1);
  Sp_Vecteur(float x1,float y1,float z1,float w1);
  Sp_Vecteur(const Sp_Vecteur &);
  // l operateur =
  Sp_Vecteur &operator=(const Sp_Vecteur &);
  // Les autres operateurs
  Sp_Vecteur operator+(const Sp_Vecteur &) const;
  Sp_Vecteur operator-(const Sp_Vecteur &) const;
  float      operator*(const Sp_Vecteur &) const;
  Sp_Vecteur operator*(const float &) const;
  // Le produit vectoriel s applique sur 3D et non 4D
  Sp_Vecteur operator^(const Sp_Vecteur &) const;
  // make the norme of vector equal to 1
  void Norme();
  // compute the norme of a vector
  float Compute_norme() const;
  // Is this vector Null
  int IsNull() const;
  // affiche un vecteur
  void Print() const;
  // Les donnees membres
  float x,y,z,w;
};


class Sp_Point
{
 public:
    Sp_Point();
    Sp_Point(float x1,float y1,float z1);
    Sp_Point(float x1,float y1,float z1,float w1);
    Sp_Point(const Sp_Point &);
        // l operateur =
    Sp_Point &operator=(const Sp_Point &);
        // les autres operateurs
    Sp_Point operator+(const Sp_Vecteur &) const;
    Sp_Point operator-(const Sp_Vecteur &) const;
    Sp_Vecteur operator-(const Sp_Point &) const;
        // l operator==
inline    int operator==(const Sp_Point &p) const 
	{
	    if(x!=p.x)
		return 0;
	    else
              if(y!=p.y)
		return 0;
              else
		if(z!=p.z)
		return 0;
	    return 1;
        }
      // la distance entre deux points
    float Distance(const Sp_Point &) const;
        // affiche un vecteur
    void Print() const;
        // Les donnees membres
    float x,y,z,w;
};





class Sp_Droite
{
 public:
  // The constructors
  Sp_Droite(Sp_Point &p,Sp_Vecteur &v);
  Sp_Droite(Sp_Point &p0,Sp_Point &p1);
  // affiche 
  void Print() const;
  // Test if the point is belonging to the straight
  int IsIn(const Sp_Point &P1) const;
  // les donnees
  Sp_Point p;
  Sp_Vecteur v;
};


class Sp_Segment
{
  public:
        // The constructors
    Sp_Segment(Sp_Point *,Sp_Point *);
    Sp_Segment(Sp_Point &,Sp_Point &);
        // Test if the point is belonging to the segment
    int IsIn(const Sp_Point &P1) const;
        // Test if the point is between
    int IsBetween(const Sp_Point &P1) const;
        // Compute Intersection between two segments
    int intersection(const Sp_Segment &seg,const Sp_Vecteur &normale,Sp_Point &) const;
        // write information
    void Print() const;
    
    
        // member data
    Sp_Point *p1;
    Sp_Point *p2;
};



class Sp_Plan
{
  public:
        // The constructors
    Sp_Plan(float a,float b,float c,float d);
    Sp_Plan(const Sp_Plan &p);
    Sp_Plan(Sp_Point &pt,Sp_Vecteur &normale);
    Sp_Plan(const Sp_Point &p1,const Sp_Point &p2,const Sp_Vecteur &v0);
        // the functions
        // Give the normale of the plan
    Sp_Vecteur Normale() const;
        // the face of a point is
    float Sign(const Sp_Point &) const;
        // Project a point on a surface
    Sp_Point Project(const Sp_Point &) const;
        // Project a vector on a surface
    Sp_Vecteur Project(const Sp_Vecteur &i1) const; 
        // Compute the distance between a point and the plan
    float Distance(const Sp_Point &p) const;
        // data
    float A,B,C,D;
};



class Sp_Sphere
{
 public:
  // les constructeurs
  Sp_Sphere(float xo,float yo,float zo,float r);
  Sp_Sphere(Sp_Point &p,float r);
  
  // les donnees
  float xo,yo,zo,r;
};




#endif
