#include <stdio.h>
#include <stdlib.h>
#include "sp_mathlin.h"
#include <math.h>

float sqr(float x)
{
  return x*x;
}


/* **********************************************************
*************** Resolution d un systeme lineaire ************
********************************************************** */


/* ************************ */
/* Les vecteurs de taille 4 */
/* ************************ */

// Les constructeurs
Sp_Vector_lin::Sp_Vector_lin(float a,float b,float c,float d)
{
  v[0]=a; v[1]=b; v[2]=c; v[3]=d;
}

// Les constructeurs
Sp_Vector_lin::Sp_Vector_lin(Sp_Plan p)
{
  v[0]=p.A; v[1]=p.B; v[2]=p.C; v[3]=p.D;
}

// affiche un Sp_Vector_lin
void Sp_Vector_lin::Print()
{
  printf("%f %f %f %f\n",v[0],v[1],v[2],v[3]);
}

// mettre a un le coef i en divisant la ligne 
void Sp_Vector_lin::Uno(int i)
{
  float coef=v[i];
  for(int j=0;j<4;j++)
    {
      v[j]/=coef;
    } 
}

/* teste vecteur nul */
/* 1:vecteur nul 0:vecteur non nul */
int Sp_Vector_lin::IsNull()
{
  if((v[0]==0)&&(v[1]==0)&&(v[2]==0))
    return 1;
  return 0;
}





/* ************************ */
/*    le systeme lineaire   */
/* ************************ */

// Les constructeurs 
Sp_System_lin::Sp_System_lin()
{
  size=0;
}

// affiche le systeme
void Sp_System_lin::Print()
{
  printf("affichage du systeme\n");
  for(int i=0;i<size;i++)
    {
      liste[i]->Print();
    }
}

void Sp_System_lin::Add(Sp_Vector_lin *a)
{
  size+=1;
  liste.push_back(a);
}

void Sp_System_lin::Echange_line(int j,int k)
{
  float tmp;
  if(j==k)
    return;
  for(int i=0;i<4;i++)
    {
      tmp=liste[j]->v[i];
      liste[j]->v[i]=liste[k]->v[i];
      liste[k]->v[i]=tmp;
    }
}


int Sp_System_lin::Find_line(int col,int lig)
{
  for(int j=lig;j<size;j++)
    {
      if(liste[j]->v[col]!=0)
        return j;
    }
  return -1;
}

int Sp_System_lin::Find_line2(int col,int lig)
{
  for(int j=lig;j>=0;j--)
    {
      if(liste[j]->v[col]!=0)
        return j;
    }
  return -1;
}

// on annule la ligne j 
void Sp_System_lin::Annule_ligne(int ligne_i,int colonne_i,int ligne_j)
{
  float f;
  f=liste[ligne_j]->v[colonne_i];
  if(f==0)
    return;
  for(int i=0;i<4;i++)
    {
      liste[ligne_j]->v[i]-=f*liste[ligne_i]->v[i];
    }
}


int Sp_System_lin::Triangulise_passe(int lig,int col)
{
  // On recherche une ligne j ou il y a un coef non nul sur la col i
  int j=Find_line(col,lig);
  if(j==-1)
    return -1;
  liste[j]->Uno(col);
  // On trouve une ligne on l echange donc
  Echange_line(lig,j);
  // On annule les lignes suivantes
  for(int k=lig+1;k<size;k++)
    {
       Annule_ligne(lig,col,k);
    }
  return 0;
  
}

int Sp_System_lin::Triangulise_passe2(int lig,int col)
{
  // si vecteur nul Pas de sol unique au Pb
  if(liste[lig]->IsNull()==1)
    return -2;
  // On recherche une ligne j ou il y a un coef non nul sur la col i
  int j=Find_line2(col,lig);
  if(j==-1)
    return -1;
  liste[j]->Uno(col);
  // On trouve une ligne on l echange donc
  Echange_line(lig,j);
  // On annule les lignes suivantes
  for(int k=lig-1;k>=0;k--)
    {
      Annule_ligne(lig,col,k);
    }
  return 0;
  
}

void Sp_System_lin::Triangulise()
{
  // le pointeur sur les lignes
  int j=0;
  // le pointeur sur les colonnes
  for(int i=0;i<2;i++)
    {
      if(Triangulise_passe(j,i)!=-1)
        {
          j+=1;
        }
    }
}

/* status=0 pas de sol sinon dans la valeur de retour */
Sp_Point Sp_System_lin::Solution(int &status)
{
  float vect[3];
  // On triangulise le systeme
  Triangulise();
  //Print();
  // on remonte le triangle
  int j=size-1;
  // i le pointeur sur les colonnes
  for(int i=2;i>=0;i--)
    {
      //printf("on triangularise\n");
      int result=Triangulise_passe2(j,i);
      //Print();
      if(result==-2)
        {
          status=0;
          return Sp_Point();
        }
      if(result==-1)
        {
          vect[i]=0;
        }
      if(result==0)
        {
          vect[i]=liste[i]->v[3];
          j--;
        }
    }
  status=1; 
  return Sp_Point(-vect[0],-vect[1],-vect[2]); 
}





/* **********************************************************
***************  Les equations du second degre **************
********************************************************** */

// le constructeur
Sp_Eq2dg::Sp_Eq2dg(float a1,float b1,float c1)
{
  a=a1; b=b1; c=c1;
}

void Sp_Eq2dg::Add(float sol)
{
  solution.push_back(sol);
}

// affiche l equation
void Sp_Eq2dg::Print()
{
    typedef list<float>::iterator iter;
    printf("liste des solutions\n");
    for(iter i=solution.begin();i!=solution.end();i++)
      {
        printf("sol:%f\n",*i);
      }
}

void Sp_Eq2dg::Solution()
{
  if(a==0)
    {
      // cas a == 0 eq du 1er dg
      if(b==0)
        return;
      else
        {
          Add(-c/b);
          return;
        }
    }
  else
    {
      // a<>0
      float delta;
      delta=b*b-4*a*c;
      if(delta<0)
        {
          // cas imaginaires
          return;
        }
     if(delta==0)
       {
         float s;
         s=-b/(2*a);
         Add(s);
         return;
       }
     // cas 2 sol
     delta=sqrt(delta);
     float s1,s2;
     s1=(-b-delta)/(2*a);
     s2=(-b+delta)/(2*a);
     Add(s1); Add(s2);
     return;
    }
}


/* **********************************************************
***************  Les solutions d intersection  **************
********************************************************** */

Intersection::Intersection(Sp_Droite &d,Sp_Sphere &s)
{
  float a,b,c;
  
  a=d.v.x+d.v.y+d.v.z;
  b=2*(d.v.x*(d.p.x-s.xo)+d.v.y*(d.p.y-s.yo)+d.v.z*(d.p.z-s.zo));
  c=sqr(d.p.x-s.xo)+sqr(d.p.y-s.yo)+sqr(d.p.z-s.zo)-sqr(s.r);
 
  Sp_Eq2dg  eq(a,b,c);
  eq.Solution();
  typedef list<float>::iterator iter;
  for(iter i=eq.solution.begin();i!=eq.solution.end();i++)
      {
        Sp_Point *p=new Sp_Point();
        *p=d.p+(d.v*(*i));
        liste_points.push_back(*p);
      }
}

Intersection::Intersection(Sp_Plan &p1,Sp_Plan &p2)
{
  Sp_Vecteur u,v,w;
  u=p1.Normale(); v=p2.Normale(); w=u^v;
  Sp_Plan plan1(p1),plan2(p2);
  if(w.x!=0)
    {
      plan1.A=plan2.A=0;
      function1(plan1,plan2,w);
    }
  else if(w.y!=0)
    {
      plan1.B=plan2.B=0;
      function1(plan1,plan2,w);
    }
  else if(w.z!=0)
    {
      plan1.C=plan2.C=0;
      function1(plan1,plan2,w);
    }
  
}

Intersection::Intersection(Sp_Droite &d,Sp_Plan &plan)
{
  float a,b,c;
  a=0; b=(d.v.x*plan.A+d.v.y*plan.B+d.v.z*plan.C); c=plan.D+plan.A*d.p.x+plan.B*d.p.y+plan.C*d.p.z;
  Sp_Eq2dg  eq(a,b,c);
  eq.Solution();
  typedef list<float>::iterator iter;
  for(iter i=eq.solution.begin();i!=eq.solution.end();i++)
    {
      Sp_Point *p1=new Sp_Point();
      *p1=d.p+(d.v*(*i));
      liste_points.push_back(*p1);
    }
}


void Intersection::function1(Sp_Plan &plan1,Sp_Plan &plan2,Sp_Vecteur &v1)
{
  // le 1er vecteur
  Sp_Vector_lin adlin(plan1);
  // le 2nd
  Sp_Vector_lin adlin2(plan2);
  // on cree un systeme et on ajoute les 2 vecteurs
  Sp_System_lin systeme;
  systeme.Add(&adlin);
  systeme.Add(&adlin2);
  int result;
  Sp_Point p_sol;
  p_sol=systeme.Solution(result);
  // on a une solution
  if(result!=0)
    {
      Sp_Droite dr(p_sol,v1);
      liste_droites.push_back(dr);
    }
}
