#include <math.h>
#include "sp_matrice.h"


// *** The constructors ***

// La classe Sp_Math est un tableau de 16 float
Sp_Mat4::Sp_Mat4()
{
  tab=new float[16];
  tab[0]=1; tab[1]=0; tab[2]=0; tab[3]=0;
  tab[4]=0; tab[5]=1; tab[6]=0; tab[7]=0;
  tab[8]=0; tab[9]=0; tab[10]=1;tab[11]=0;
  tab[12]=0;tab[13]=0;tab[14]=0;tab[15]=1;
}


Sp_Mat4 Sp_Mat4::put() const
{
  Sp_Mat4 tmp;
  tmp.tab[0]=tab[0];tmp.tab[1]=tab[4];tmp.tab[2]=tab[8];tmp.tab[3]=tab[12];
  tmp.tab[4]=tab[1];tmp.tab[5]=tab[5];tmp.tab[6]=tab[9];tmp.tab[7]=tab[13];
  tmp.tab[8]=tab[2];tmp.tab[9]=tab[6];tmp.tab[10]=tab[10];tmp.tab[11]=tab[14];
  tmp.tab[12]=tab[3];tmp.tab[13]=tab[7];tmp.tab[14]=tab[11];tmp.tab[15]=tab[15]; 
  return tmp;
}


Sp_Mat4 &Sp_Mat4::operator=(const Sp_Mat4 &t)
{
  tab[0]=t.tab[0];  tab[1]=t.tab[1];  tab[2]=t.tab[2];  tab[3]=t.tab[3];
  tab[4]=t.tab[4];  tab[5]=t.tab[5];  tab[6]=t.tab[6];  tab[7]=t.tab[7];
  tab[8]=t.tab[8];  tab[9]=t.tab[9];  tab[10]=t.tab[10];tab[11]=t.tab[11];
  tab[12]=t.tab[12];tab[13]=t.tab[13];tab[14]=t.tab[14];tab[15]=t.tab[15];
  return *this;
}

void Sp_Mat4::Id()
{
  //  tab=new float[16];
  tab[0]=1; tab[1]=0; tab[2]=0; tab[3]=0;
  tab[4]=0; tab[5]=1; tab[6]=0; tab[7]=0;
  tab[8]=0; tab[9]=0; tab[10]=1;tab[11]=0;
  tab[12]=0;tab[13]=0;tab[14]=0;tab[15]=1;
}

Sp_Mat4::Sp_Mat4(float *t)
{
  int i;
  tab=new float[16];
  for(i=0;i<16;i++)
    {
      tab[i]=t[i];
    }
}

void Sp_Mat4::Set(float *t)
{
  int i;
  for(i=0;i<16;i++)
    {
      tab[i]=t[i];
    }
}

void Sp_Mat4::Set(const Sp_Vecteur &i,const Sp_Vecteur &j,const Sp_Vecteur &k,const Sp_Point &O)
{
    tab[0]=i.x; tab[4]=i.y; tab[8]=i.z; tab[12]=0;
    tab[1]=j.x; tab[5]=j.y; tab[9]=j.z; tab[13]=0;
    tab[2]=k.x; tab[6]=k.y; tab[10]=k.z; tab[14]=0;
    tab[3]=O.x; tab[7]=O.y; tab[11]=O.z; tab[15]=1;
}



void Sp_Mat4::Get(float *t)
{
  int i;
  for(i=0;i<16;i++)
   {
     t[i]=tab[i];
   }
}

void Sp_Mat4::Trans(float x,float y,float z)
{
  tab[0]=1;  tab[1]=0;   tab[2]=0;   tab[3]=x;
  tab[4]=0;  tab[5]=1;   tab[6]=0;   tab[7]=y;
  tab[8]=0;  tab[9]=0;   tab[10]=1;  tab[11]=z;
  tab[12]=0; tab[13]=0;  tab[14]=0;  tab[15]=1;
}




// matrice de rotation autour de l axe X de l angle r
void	Sp_Mat4::SetX(float r)
{
  tab[0]=1;  tab[1]=0;      tab[2]=0;       tab[3]=0;
  tab[4]=0;  tab[5]=cos(r); tab[6]=-sin(r); tab[7]=0;
  tab[8]=0;  tab[9]=sin(r); tab[10]=cos(r); tab[11]=0;
  tab[12]=0; tab[13]=0;     tab[14]=0;      tab[15]=1;
}

void	Sp_Mat4::SetY(float s)
{
  tab[0]=cos(s); tab[1]=0;  tab[2]=-sin(s);  tab[3]=0;
  tab[4]=0;      tab[5]=1;  tab[6]=0;        tab[7]=0;
  tab[8]=sin(s); tab[9]=0;  tab[10]=cos(s);  tab[11]=0;
  tab[12]=0;     tab[13]=0; tab[14]=0;       tab[15]=1;
}

void	Sp_Mat4::SetZ(float t)
{
  tab[0]=cos(t);  tab[1]=-sin(t);  tab[2]=0;  tab[3]=0;
  tab[4]=sin(t);  tab[5]=cos(t);   tab[6]=0;  tab[7]=0;
  tab[8]=0;       tab[9]=0;        tab[10]=1; tab[11]=0;
  tab[12]=0;      tab[13]=0;       tab[14]=0; tab[15]=1;
}


Sp_Mat4 Sp_Mat4::operator*(const Sp_Mat4 &b) const
{
  Sp_Mat4 tmp;

  tmp.tab[0]=tab[0]*b.tab[0] + tab[1]*b.tab[4] + tab[2]*b.tab[8] + tab[3]*b.tab[12];
  tmp.tab[1]=tab[0]*b.tab[1] + tab[1]*b.tab[5] + tab[2]*b.tab[9] + tab[3]*b.tab[13];
  tmp.tab[2]=tab[0]*b.tab[2] + tab[1]*b.tab[6] + tab[2]*b.tab[10]+ tab[3]*b.tab[14];
  tmp.tab[3]=tab[0]*b.tab[3] + tab[1]*b.tab[7] + tab[2]*b.tab[11]+ tab[3]*b.tab[15];
  
  tmp.tab[4]=tab[4]*b.tab[0] + tab[5]*b.tab[4] + tab[6]*b.tab[8]+  tab[7]*b.tab[12];
  tmp.tab[5]=tab[4]*b.tab[1] + tab[5]*b.tab[5] + tab[6]*b.tab[9]+  tab[7]*b.tab[13];
  tmp.tab[6]=tab[4]*b.tab[2] + tab[5]*b.tab[6] + tab[6]*b.tab[10]+ tab[7]*b.tab[14];
  tmp.tab[7]=tab[4]*b.tab[3] + tab[5]*b.tab[7] + tab[6]*b.tab[11]+ tab[7]*b.tab[15];

  tmp.tab[8]=tab[8]*b.tab[0] + tab[9]*b.tab[4] + tab[10]*b.tab[8]+ tab[11]*b.tab[12];
  tmp.tab[9]=tab[8]*b.tab[1] + tab[9]*b.tab[5] + tab[10]*b.tab[9]+ tab[11]*b.tab[13];
  tmp.tab[10]=tab[8]*b.tab[2]+ tab[9]*b.tab[6] + tab[10]*b.tab[10]+tab[11]*b.tab[14];
  tmp.tab[11]=tab[8]*b.tab[3]+ tab[9]*b.tab[7] + tab[10]*b.tab[11]+tab[11]*b.tab[15];

  tmp.tab[12]=tab[12]*b.tab[0]+tab[13]*b.tab[4]+ tab[14]*b.tab[8]+ tab[15]*b.tab[12];
  tmp.tab[13]=tab[12]*b.tab[1]+tab[13]*b.tab[5]+ tab[14]*b.tab[9]+ tab[15]*b.tab[13];
  tmp.tab[14]=tab[12]*b.tab[2]+tab[13]*b.tab[6]+ tab[14]*b.tab[10]+tab[15]*b.tab[14];
  tmp.tab[15]=tab[12]*b.tab[3]+tab[13]*b.tab[7]+ tab[14]*b.tab[11]+tab[15]*b.tab[15];
  
  return tmp;
}

Sp_Mat4 Sp_Mat4::Inverse() const
{
  Sp_Mat4 tmp;
  // on prend la transpose de la mat 3x3
  tmp.tab[0]=tab[0]; tmp.tab[1]=tab[4]; tmp.tab[2]=tab[8];
  tmp.tab[4]=tab[1]; tmp.tab[5]=tab[5]; tmp.tab[6]=tab[9];
  tmp.tab[8]=tab[2]; tmp.tab[9]=tab[6]; tmp.tab[10]=tab[10];
  tmp.tab[3]= -(tmp.tab[0]*tab[3]+tmp.tab[1]*tab[7]+tmp.tab[2]*tab[11])/tab[15];
  tmp.tab[7]= -(tmp.tab[4]*tab[3]+tmp.tab[5]*tab[7]+tmp.tab[6]*tab[11])/tab[15];
  tmp.tab[11]= -(tmp.tab[8]*tab[3]+tmp.tab[9]*tab[7]+tmp.tab[10]*tab[11])/tab[15];
  tmp.tab[12]=tab[12]; tmp.tab[13]=tab[13]; tmp.tab[14]=tab[14]; tmp.tab[15]=1/tab[15];
  return tmp;
}

void Sp_Mat4::Print()
{
  int i;
  printf("une Sp_Mat4:\n");
  for(i=0;i<16;i+=4)
    {
      printf("a%02d:%03f ",i,tab[i]);
      printf("a%02d:%03f ",i+1,tab[i+1]);
      printf("a%02d:%03f ",i+2,tab[i+2]);
      printf("a%02d:%03f\n",i+3,tab[i+3]);
    }
}
