#include "cobj.h"
#include "face.h"
#include "ImageBuffer.h"
/* -------------------------------------------------------------- */
/*                              Sp_Obj                            */
/* -------------------------------------------------------------- */

Sp_Obj::Sp_Obj()
{
  Name="Unknown";
  father=0;
  adm_obj=0;
  trans=new Sp_LTrans();
  
      //coord=new Sp_Mat4;
      //coord->Id();
}

Sp_Obj::Sp_Obj(char *name,Sp_Adm *adm3d,Sp_Obj *your_father,int priority)
{
    Name=name;
    adm_obj=adm3d;
    father=your_father;
    trans=new Sp_LTrans();
  
        //   coord=new Sp_Mat4;
    coord_camera=new Sp_Mat4;
        // coord->Id();
    switch(priority)
    {
        case SP_FIRST_3DPRIORITY:
            adm3d->draw_first.push_back(this);
            break;
        case SP_LAST_3DPRIORITY:
             adm3d->draw_last.push_back(this);
            break;
        case SP_NORMAL_3DPRIORITY:
        default:
            father->Add_Son(this);
            break;
    }
}

/* le destructeur utilisera rem son */

void Sp_Obj::Add_Son(Sp_Obj *son)
{
  list_Sons.push_back(son);
}

void Sp_Obj::Rem_Son(Sp_Obj *son)
{
  list_Sons.remove(son);
}

void Sp_Obj::Add_Entity(Sp_EntityBase *entity)
{
  list_entities.push_back(entity);
}

void Sp_Obj::Rem_Entity(Sp_EntityBase *entity)
{
  list_entities.remove(entity);
}

void Sp_Obj::Go_Up()
{
  // debut de WORK
  Work();
  // fin de WORK
  typedef list<Sp_Obj *>::iterator iter_Sp_Obj;
  // On recupere le pere
  if(father==0)
    return;
  glPushMatrix();
  glMultMatrixf(trans->result.Inverse().put().tab);
  glGetFloatv(GL_MODELVIEW_MATRIX,father->coord_camera->tab);
  father->Go_Down(this);
  father->Go_Up();
  glPopMatrix();
}

void Sp_Obj::Go_Down(Sp_Obj *obj)
{
  // On parcourt la liste des fils
  typedef list<Sp_Obj *>::iterator iter_Sp_Obj;
  for(iter_Sp_Obj i=list_Sons.begin();i!=list_Sons.end();i++)
    {
      if((*i)==obj)
        continue;
      glPushMatrix();
      glMultMatrixf((*i)->trans->result.put().tab);
      glGetFloatv(GL_MODELVIEW_MATRIX,(*i)->coord_camera->tab);
      // deb de WORK
      (*i)->Work();
      // fin de WORK
      (*i)->Go_Down(0);
      glPopMatrix();
    }
}

void Sp_Obj::Work()
{
  // printf("work de %s\n",Name.data());
}


/* -------------------------------------------------------------- */
/*                              Sp_Camera                         */
/* -------------------------------------------------------------- */
Sp_Camera::Sp_Camera(char *name,Sp_Adm *adm3d,Sp_Obj *your_father):Sp_Obj(name,adm3d,your_father,SP_NORMAL_3DPRIORITY)
{
  adm_obj->Add_Camera(this);
}

Sp_Camera::~Sp_Camera()
{
  adm_obj->Rem_Camera(this);
}


/* -------------------------------------------------------------- */
/*                              Sp_Obj3D                          */
/* -------------------------------------------------------------- */
Sp_Obj3d::Sp_Obj3d(char *name,Sp_Adm *gest3d,Sp_Obj *ton_pere,int priority):Sp_Obj(name,gest3d,ton_pere,priority)
{
}


/* -------------------------------------------------------------- */
/*                              Sp_Adm                           */
/* -------------------------------------------------------------- */
Sp_Camera *cam_default;


//   Le constructeur
Sp_Adm::Sp_Adm(char *name):Sp_Adm_Time()
{
  Name=name;
  adm_obj=this;
  father=0;
      //coord=new Sp_Mat4;
  coord_camera=new Sp_Mat4;
      // coord->Id();
  // La gestion des textures
  texture_indice=0;
  cam_default=new Sp_Camera("Camera par default",this,this);
  // On la rend active
  camera_active=cam_default;
}



//   Le destructeur
Sp_Adm::~Sp_Adm()
{
  delete cam_default;
}

int Sp_Adm::Load_texture(char *filename)
{
  ImageBuffer img(filename);
  img.toRGBA();
  
  int tex=Add_Texture((char*)img.getData(),img.getWidth(),img.getHeight());
  return tex;
}

int Sp_Adm::Add_Texture(char *texture,int w,int h)
{
    glGenTextures(1,(GLuint *)&texture_indice);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, texture_indice);
    gluBuild2DMipmaps(GL_TEXTURE_2D,4,w,h,GL_RGBA,GL_UNSIGNED_BYTE,texture);
    return texture_indice;
}

void Sp_Adm::Add_Camera(Sp_Camera *camera)
{
    liste_cameras.push_back(camera);
}


void Sp_Adm::Rem_Camera(Sp_Camera *camera)
{
  liste_cameras.remove(camera);
}

void Sp_Adm::DrawScene()
{
        // this function must be called once at each frame
        // it computes the time elapsed with the last frame
        // it is a function from the time server
    FrameTick();
    
    *coord_camera=ComputeObjtoGest3d(camera_active);
    
    ComputeObjtoCam();
    
    typedef list<Sp_Obj *>::iterator iter_Sp_Obj;
        // we draw first the first list of objects
    for(iter_Sp_Obj i=draw_first.begin();i!=draw_first.end();i++){
        glPushMatrix();
        *(*i)->coord_camera=(*coord_camera)*ComputeObjtoGest3d(*i);
        glLoadMatrixf((*i)->coord_camera->put().tab);
        (*i)->Work();
        glPopMatrix();
    }

    glPushMatrix();
    Render();
    glPopMatrix();

        // we draw last list of objects
    for(iter_Sp_Obj i=draw_last.begin();i!=draw_last.end();i++){
        glPushMatrix();
        *(*i)->coord_camera=(*coord_camera)*ComputeObjtoGest3d(*i);
        glLoadMatrixf((*i)->coord_camera->put().tab);
        (*i)->Work();
        glPopMatrix();
    }

}


Sp_Mat4 Sp_Adm::ComputeObjtoGest3d(Sp_Obj *pt)
{
    Sp_Mat4 rep;
    pt->trans->Compute();
    rep=pt->trans->result.Inverse();
    while(pt->father!=this){
        pt=pt->father;
        pt->trans->Compute();
        rep=rep*(pt->trans->result.Inverse());
    }
    return rep;
}

void Sp_Obj::ComputeObjtoCam()
{
#ifdef DEBUG
    printf("work de %s\n",Name.data());
    coord_camera->Print();
#endif    
        // On parcourt la liste des fils
  typedef list<Sp_Obj *>::iterator iter_Sp_Obj;
  for(iter_Sp_Obj i=list_Sons.begin();i!=list_Sons.end();i++)
  {
      (*i)->trans->Compute();
      *((*i)->coord_camera)=(*coord_camera)*((*i)->trans->result);
      (*i)->ComputeObjtoCam();
  }
}
void Sp_Obj::Render()
{
  // We are going through the liste of Sons
  typedef list<Sp_Obj *>::iterator iter_Sp_Obj;
  glLoadMatrixf(coord_camera->put().tab);
  Work();
  for(iter_Sp_Obj i=list_Sons.begin();i!=list_Sons.end();i++){
      (*i)->Render();
  }
  
}

