#include "sp_circuit.h"


/* *******************************************
****************  A CIRCUIT ******************
******************************************* */
Sp_CircuitC::Sp_CircuitC()
{
    index=0;
}

void Sp_CircuitC::Add_Face(Sp_FaceC *face)
{
    face->num_face=index;
    list_faces.push_back(face);
    index++;
}


// compute the direct neighbours of the face
void Sp_CircuitC::ComputeNeighbours()
{
    typedef vector<Sp_FaceC *>::iterator iter;
    typedef vector<Sp_SegmentC *>::iterator iter2;
    typedef vector<Sp_Point *>::iterator iter3;
	int fg=0;
	int face_nm=0;

    if(Load()!=0){
        return ;
    }
    

    
        // on parcourt les faces
    for(iter ifa=list_faces.begin();ifa!=list_faces.end();ifa++)
    {
            // on parcourt les segments
        for(iter2 j=(*ifa)->list_segments.begin();j!=(*ifa)->list_segments.end();j++)
        {
			// we have two points
            Sp_Point &p1=*(*j)->p1;
            Sp_Point &p2=*(*j)->p2;
                // on reparcourt les faces
            for(iter k=list_faces.begin();k!=list_faces.end();k++)
            {
                if(k==ifa)
                    continue;
                    // we look into the points
                for(iter3 l=(*k)->list_points.begin();l!=(*k)->list_points.end();l++)
                {
                    if( (p1==*(*l)) || (p2==*(*l)) )
                    {
                        (*j)->Add_Face(*k);
                    }
                }
            }
            
        }
        
    }
    Save();
}

void Sp_CircuitC::Save()
{
    FILE *File;
    File=fopen("precomputations.sav","wb");

    typedef vector<Sp_FaceC *>::iterator iter;
    typedef vector<Sp_SegmentC *>::iterator iter2;
    typedef list<Sp_FaceC *>::iterator iter3;


        // on parcourt les faces
    for(iter ifa=list_faces.begin();ifa!=list_faces.end();ifa++)
    {
            // on parcourt les segments
        for(iter2 j=(*ifa)->list_segments.begin();j!=(*ifa)->list_segments.end();j++)
        {
            int CHUNKID_SEGEMNT=1111;
            int numface2;
                // The ID
            fwrite((void *)&CHUNKID_SEGEMNT,1,4,File);
                // the size
            int size;
            size=4+4+4*((*j)->list_faces.size());
            fwrite((void *)&size,1,4,File);
            
                // on pose les indices de la liste
                // on parcourt les faces voisines
            for(iter3 k=(*j)->list_faces.begin();k!=(*j)->list_faces.end();k++)
            {
                numface2=(*k)->num_face;
                fwrite((void *)&numface2,1,4,File);
            }
            
        }
    }
    fclose(File);
 
}


int Sp_CircuitC::Load()
{
    typedef vector<Sp_FaceC *>::iterator iter;
    typedef vector<Sp_SegmentC *>::iterator iter2;
    FILE *File;
    File=fopen("precomputations.sav","rb");
    if(File==0){
        return 0;
    }
        // on parcourt les faces
    for(iter ifa=list_faces.begin();ifa!=list_faces.end();ifa++)
    {
            // on parcourt les segments
        for(iter2 j=(*ifa)->list_segments.begin();j!=(*ifa)->list_segments.end();j++)
        {
            int CHUNKID_SEGEMNT;
                // The ID
            fread((void *)&CHUNKID_SEGEMNT,1,4,File);
            if(CHUNKID_SEGEMNT!=1111)
            {
                printf("file of precomputations corrupted\n");
                exit(0);
            }
                // the size
            int size;
            fread((void *)&size,1,4,File);
            size=(size-8)/4;

                    // on parcourt les faces voisines
            int numface=0;
            for(int k=0;k<size;k++)
            {
                fread((void *)&numface,1,4,File );
                (*j)->Add_Face(list_faces[numface]);
            }
        }
    }
    fclose(File);
    return 1;
}



/* *******************************************
*****************  A  FACE  ******************
******************************************* */
Sp_FaceC::Sp_FaceC(float *p1,float *p2,float *p3)
{
    index=0;
        // we're building points
    Sp_Point *P1;
    Sp_Point *P2;
    Sp_Point *P3;
        // printf("building face\n");
    P1=new Sp_Point(p1[0],p1[1],p1[2]);//P1->Print();
    list_points.push_back(P1);
    P2=new Sp_Point(p2[0],p2[1],p2[2]);//P2->Print();
    list_points.push_back(P2);
    P3=new Sp_Point(p3[0],p3[1],p3[2]);//P3->Print();
    list_points.push_back(P3);
        // we're buildings the segment
    Sp_SegmentC *s;
    s=new Sp_SegmentC(P1,P2);
    Add_Seg(s);
        //list_segments.push_back(s);
    s=new Sp_SegmentC(P2,P3);
    Add_Seg(s);
        //list_segments.push_back(s);
    s=new Sp_SegmentC(P3,P1);
    Add_Seg(s);
        //list_segments.push_back(s);

    //Precalculate Normal+center
    Calc_Normale();
    Calc_Center();
}

void Sp_FaceC::Add_Seg(Sp_SegmentC *seg)
{
    list_segments.push_back(seg);
    seg->num_seg=index;
    index++;
}





void Sp_FaceC::Calc_Normale()
{
  Sp_Vecteur u,v;
  u=*list_points[1]-*list_points[0];
  v=*list_points[2]-*list_points[0];
  norm=u^v;
}

void Sp_FaceC::Calc_Center()
{
    Sp_Point result;
    result.x=list_points[0]->x+list_points[1]->x+list_points[2]->x;
    result.y=list_points[0]->y+list_points[1]->y+list_points[2]->y;
    result.z=list_points[0]->z+list_points[1]->z+list_points[2]->z;
    result.x/=3; result.y/=3; result.z/=3;
    center=result;
}



int Sp_FaceC::IsIn(Sp_Point &p)
{
    typedef vector<Sp_SegmentC *>::iterator iter;
    Sp_Vecteur normale=Normale();
    Sp_Point P=Center();
    for(iter i=list_segments.begin();i!=list_segments.end();i++)
    {
        if((*i)->SameFace(P,p,normale)<0){            
            return 0;
        }
        
    }
    return 1;
}


Sp_Point Sp_FaceC::Project(Sp_Point &p)
{
    Sp_Vecteur v=Normale();
    Sp_Plan plan(*list_points[0],v);
    return plan.Project(p);
}

Sp_Vecteur Sp_FaceC::Project(Sp_Vecteur &v1)
{
    Sp_Vecteur v=Normale();
    Sp_Plan plan(*list_points[0],v);
    return plan.Project(v1);
}


int Sp_FaceC::WhichCut(Sp_Point &P,Sp_Point &P1,Sp_Point &pt_result,Sp_FaceC **new_face)
{
    typedef vector<Sp_SegmentC *>::iterator iter;
    typedef list<Sp_FaceC *>::iterator iter2;
    Sp_Vecteur normale=Normale();
    
    Sp_Point result;
    
    if(this!=*new_face){
        printf("gros pb\n");
        exit(0);
    }
    
    Sp_Segment seg(P,P1);
    
    for(iter i=list_segments.begin();i!=list_segments.end();i++)
    {
        if((*i)->intersection(seg,normale,result)!=0)
        {
            pt_result=result;
            Sp_LFaceC_Tested find(this);
            *new_face=find.FindFace(P1);
            if(*new_face==0)
                printf("lolo\n");
            return 1;
        }
    }
    printf("nb de segements %d\n",list_segments.size());
    Print2();
        //result.Print();
    exit(0);
    return 0;
}



void Sp_FaceC::Print()
{
        // les segements
    printf("on affiche une face\n");   
    (*list_points[0]).Print();
    (*list_points[1]).Print();
    (*list_points[2]).Print();
        // on affiche ses voisines
    typedef vector<Sp_SegmentC *>::iterator iter;
        // on parcourt les segments
    for(iter j=list_segments.begin();j!=list_segments.end();j++)
    {
        (*j)->Print();
        
    }
    
}
void Sp_FaceC::Print2()
{
        // les sgements
    printf("on affiche une face voisine\n");   
    (*list_points[0]).Print();
    (*list_points[1]).Print();
    (*list_points[2]).Print();
}




/* *******************************************
***************** A SEGMENT ******************
******************************************* */

Sp_SegmentC::Sp_SegmentC(Sp_Point *pp1,Sp_Point *pp2)
        :Sp_Segment(pp1,pp2)
{

}

void Sp_SegmentC::Add_Face(Sp_FaceC *face)
{
  list_faces.push_back(face);
}



// Compute the plan which contains the normale and the point
Sp_Plan Sp_SegmentC::Compute_Plan(Sp_Vecteur &normale)
{
  return Sp_Plan(*p1,*p2,normale);
}



// Are the points in the same side from plane
// yes if dot >= 0 else no
float Sp_SegmentC::SameFace(Sp_Point &p1,Sp_Point &p2,Sp_Vecteur &normale)
{
  Sp_Plan p=Compute_Plan(normale);
  return p.Sign(p1)*p.Sign(p2);
}

void Sp_SegmentC::Print()
{
    Sp_Segment::Print();    
    typedef list<Sp_FaceC *>::iterator iter;
    for(iter k=list_faces.begin();k!=list_faces.end();k++)
    {
        (*k)->Print2();
    }
}


void Sp_SegmentC::Save(FILE *fichier)
{
    int CHUNKID_SEGEMNT=1111;
    int numface2;
        // The ID
    fwrite((void *)&CHUNKID_SEGEMNT,1,4,fichier);
        // the size
    int size;
    size=4+4+4*list_faces.size();
    fwrite((void *)&size,1,4,fichier);
    
        // on pose les indices de la liste
    typedef list<Sp_FaceC *>::iterator iter;
        // on parcourt les faces voisines
    for(iter j=list_faces.begin();j!=list_faces.end();j++)
    {
        numface2=(*j)->num_face;
        fwrite((void *)&numface2,1,4,fichier);
    }
    

}

int Sp_SegmentC::Load(FILE *fichier)
{
    int CHUNKID_SEGEMNT;
        // The ID
    fread((void *)&CHUNKID_SEGEMNT,1,4,fichier);
    if(CHUNKID_SEGEMNT!=1111)
    {
        printf("file of precomputations corrupted\n");
        exit(0);
    }
        // the size
    int size;
    fread((void *)&size,1,4,fichier);
    size=(size-8)/4;
    return size;
    
}







/* *******************************************
** a class to find on which face a point is **
******************************************* */
Sp_FaceC_Tested::Sp_FaceC_Tested(Sp_FaceC *f)
{
    face=f;
        // we are not yet tested this face
    is_test=0;
}



Sp_LFaceC_Tested::Sp_LFaceC_Tested(Sp_FaceC *f)
{
    Sp_FaceC_Tested  *first_element=new Sp_FaceC_Tested(f);
    list_faces.push_back(first_element);
    
}

Sp_FaceC *Sp_LFaceC_Tested::FindFace(Sp_Point &p)
{
    Sp_FaceC_Tested *facet;
    facet=FindFirstFaceNotTested();
    if(facet==0){
        return 0;
    }
    
    if(facet->face->IsIn(p)!=0)
        return facet->face;
    facet->is_test=1;
    Add_Neighbours(facet->face);
    return FindFace(p);
}

Sp_FaceC_Tested *Sp_LFaceC_Tested::FindFirstFaceNotTested()
{
    typedef list<Sp_FaceC_Tested *>::iterator iter;
    for(iter k=list_faces.begin();k!=list_faces.end();k++)
    {
        if((*k)->is_test==0)
            return *k;
    }
    return 0;
}

int Sp_LFaceC_Tested::Add_Face(Sp_FaceC *f)
{
        // we're adding only this face if it is not yet in the list
    typedef list<Sp_FaceC_Tested *>::iterator iter;
    for(iter k=list_faces.begin();k!=list_faces.end();k++){
        if((*k)->face==f)
            return 0;
    }
        // not yet in
    Sp_FaceC_Tested  *first_element=new Sp_FaceC_Tested(f);
    list_faces.push_back(first_element);
    return 1;
}

void Sp_LFaceC_Tested::Add_Neighbours(Sp_FaceC *f)
{
        // on parcoure les segments voisins
    typedef vector<Sp_SegmentC *>::iterator iter;
        // on parcourt les segments
    for(iter j=f->list_segments.begin();j!=f->list_segments.end();j++)
    {
        typedef list<Sp_FaceC *>::iterator iter2;
        for(iter2 k=(*j)->list_faces.begin();k!=(*j)->list_faces.end();k++)
        {
            Add_Face(*k);
        }
    }
   
}
