#ifndef __CIRCUIT_H
#define __CIRCUIT_H

#include <string>
#include <list>
#include <vector>
#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>
using namespace std;



#include "sp_matrice.h"
#include "sp_math.h"
#include "sp_mathlin.h"



// Nothing Optimised

class Sp_SegmentC;
class Sp_FaceC;
class Sp_CircuitC;
class Sp_FaceC_Tested;






/* ******************************************
  **  a circuit is a set of  faces **
 ***************************************** */

class Sp_CircuitC
{
  public:
        // the constructors
    Sp_CircuitC();
        // Add and remove face from the list
    void Add_Face(Sp_FaceC *);
    void Rem_Face(Sp_FaceC *);
        // it computes the faces neighboring a segment
    void ComputeNeighbours();
        // member data
    vector<Sp_FaceC *> list_faces;

        // save the precomputation
    void Save();
    int Load();
    
  private:
    int index;
    
};










/* ***************************************
   ** a face is a closed outline **
*************************************** */
class Sp_FaceC
{
  private:
	Sp_Vecteur norm;
	Sp_Point center;
	void Calc_Center(void);
	void Calc_Normale(void);

  public:
        // member functions
        // the constructors
    Sp_FaceC(float *p1,float *p2,float *p3);
    
        // Compute the normal of the face
    Sp_Vecteur Normale(){return norm;}
    
        // Compute the center of a face
    Sp_Point Center(){return center;}
    
        // Is the point in the face
    int IsIn(Sp_Point &);
    
        // Project the point on the face
    Sp_Point Project(Sp_Point &);

        // projet a vecteur on the face
    Sp_Vecteur Project(Sp_Vecteur &);

        // find the segment which is cut by tne new position
    int WhichCut(Sp_Point &P,Sp_Point &P1,Sp_Point &result,Sp_FaceC **new_face);
    
        // Print information for a face
    void Print();
    void Print2();


    void Add_Seg(Sp_SegmentC *);
    void Rem_Seg(Sp_SegmentC *);
    
        // member data
        // The list of points which constitute the face
        // they are ordonned with sense (clock wise)
    vector<Sp_Point *> list_points;
    
        // the list of segment
        // a segment (2 points) allows to know the (other) faces
        // which touch him. The points of a segment belong to list_points
    vector<Sp_SegmentC *> list_segments;

  public:
    int num_face;
    int index;
};





/* *****************************************
** a segment allows to know the neighbour **
***************************************** */

class Sp_SegmentC: public Sp_Segment
{
  public:
        // Members Functions
    Sp_SegmentC(Sp_Point *,Sp_Point *);

        // Add an remove a face
    void Add_Face(Sp_FaceC *);


        // Compute the plan
    Sp_Plan Compute_Plan(Sp_Vecteur &normale);

        //  Test if the two points are on the same side 
    float SameFace(Sp_Point &p1,Sp_Point &p2,Sp_Vecteur &normale);

        // see the Sp_Segement
    void Print();
    
    void Save(FILE *);
    int Load(FILE *);
    
    
        // The list of neighbours
    list<Sp_FaceC *> list_faces;

  public:
    int num_seg;
};





/* *******************************************
** a class to find on which face is a point **
******************************************* */
class Sp_FaceC_Tested
{
  public:
        // The members functions
    Sp_FaceC_Tested(Sp_FaceC *);

        //The Data Members
    int is_test;
    Sp_FaceC *face;
};

class Sp_LFaceC_Tested
{
  public:
        // the members functions
    Sp_LFaceC_Tested(Sp_FaceC *);
        // Find the Face which contains this point
    Sp_FaceC *FindFace(Sp_Point &);

  protected:
        // Find the fist face not tested
    Sp_FaceC_Tested *FindFirstFaceNotTested();
        // Add a face if it is not in the liste
    int Add_Face(Sp_FaceC *);
        // Add the neighbours of a face in the liste
    void Add_Neighbours(Sp_FaceC *);
    
        // the members data
  public:
    list<Sp_FaceC_Tested *> list_faces;
};




#endif
